unit Backdrop;
//Modified Gradient Backdrop. RefreshEvent needs to be handled and paint called
//when it ocurrs by the calling class. When the Enabled property is set to False
//no redrawing on the bitmap occurs so the calling class must redraw it. Colors
//not used tint the bitmap, the properties Change r,g or b are set to false if
//not used. Otherwise they are changed from 0 to 255 to affect the gradient.
//Likewise the Start color value is meaningfull only if the color is not used ie
//set to false.
interface

uses
  Windows, Messages, Graphics, Classes, Controls;

type
  TFillDirection = (drLeftRight, drRightLeft, drUpDown, drDownUp);
  TRefreshEvent = procedure of object;
  TBackdrop = class(TPersistent)
  private
    { Private declarations }
    FBitMap: TBitMap;
    FStartR: Byte;
    FStartG: Byte;
    FStartB: Byte;
    FChangeR: Boolean;
    FChangeG: Boolean;
    FChangeB: Boolean;
    FDirection: TFillDirection;
    FEnabled: Boolean;
    FRefreshEvent: TRefreshEvent;
    procedure FillBackdrop;
    function RectLeftRight(i, w2: Integer; w: Real): TRect;
    function RectRightLeft(i, w2: Integer; w: Real): TRect;
    function RectUpDown(i, w2: Integer; w: Real): TRect;
    function RectDownUp(i, w2: Integer; w: Real): TRect;
    procedure ChangeDrawR(NewVal: Boolean);
    procedure ChangeDrawG(NewVal: Boolean);
    procedure ChangeDrawB(NewVal: Boolean);
    procedure ChangeStartR(NewVal: Byte);
    procedure ChangeStartG(NewVal: Byte);
    procedure ChangeStartB(NewVal: Byte);
    procedure ChangeDirection(NewVal: TFillDirection);
    procedure ChangeEnabled(NewVal: Boolean);
  protected
    { Protected declarations }
    procedure RefreshNeeded;
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
    procedure Paint(BitMap: TBitMap);
  published
    { Published declarations }
    property ChangeRColour: Boolean Read FChangeR Write ChangeDrawR;
    property ChangeGColour: Boolean Read FChangeG Write ChangeDrawG;
    property ChangeBColour: Boolean Read FChangeB Write ChangeDrawB;
    property Direction: TFillDirection Read FDirection Write ChangeDirection;
    property StartR: Byte Read FStartR Write ChangeStartR;
    property StartG: Byte Read FStartG Write ChangeStartG;
    property StartB: Byte Read FStartB Write ChangeStartB;
    property Enabled: Boolean read FEnabled write ChangeEnabled;
    property RefreshEvent: TRefreshEvent read FRefreshEvent write FRefreshEvent;
  end;

implementation

constructor TBackdrop.Create;
begin
  FDirection := drLeftRight;
  FChangeR := True;
  FChangeG := True;
  FChangeB := True;
  FEnabled := False;
end;

destructor TBackdrop.Destroy;
begin
  inherited Destroy;
end;

procedure TBackdrop.Paint(BitMap: TBitMap);
begin
  if FEnabled then
  begin
    FBitmap := BitMap;
    FillBackdrop;
  end;
end;

procedure TBackdrop.FillBackdrop;
var
  W: Real;
  W2: Integer;
  R,G,B: Byte;
  i: Integer;
  RectArea: TRect;
begin
  // Calculate the size of each strip of colour (there will be 256 of them)
  if FDirection <= drRightLeft then
    W := FBitMap.Width / 256
  else
    W := FBitMap.Height / 256;
  W2 := Round(W + 0.5);
  // The starting values for each colour.  If a colour is not used then the
  // starting value will tint the entire control in a funky fashion
  R := FStartR;
  G := FStartG;
  B := FStartB;
  // Display 256 different coloured rectangles along the component to display
  // a smooth gradient
  for i := 0 to 255 do
  begin
    if FChangeR then R := i;
    if FChangeG then G := i;
    if FChangeB then B := i;
    FBitMap.Canvas.Brush.Color := RGB(R, G, B);
    // Figure out which part of the image to fill
    case FDirection of
      drLeftRight: RectArea := RectLeftRight(i,w2,w);
      drRightLeft: RectArea := RectRightLeft(i,w2,w);
      drUpDown   : RectArea := RectUpDown(i,w2,w);
      drDownUp   : RectArea := RectDownUp(i,w2,w);
    end;
    FBitMap.Canvas.FillRect(RectArea);
  end;
end;

function TBackdrop.RectLeftRight(i, w2: Integer; w: Real): TRect;
begin
  Result := Bounds(Round(w * i), 0, w2, FBitMap.Height);
end;

function TBackdrop.RectRightLeft(i, w2: Integer; w: Real): TRect;
begin
  Result := Bounds(Round(w * (255 - i)), 0, w2, FBitMap.HEIGHT);
end;

function TBackdrop.RectUpDown(i, w2: Integer; w: Real): TRect;
begin
  Result := Bounds(0, Round(w * i), FBitMap.Width, w2);
end;

function TBackdrop.RectDownUp(i, w2: Integer; w: Real): TRect;
begin
  Result := Bounds(0, Round(w * (255 - i)), FBitMap.Width, w2);
end;

procedure TBackdrop.ChangeDrawR(NewVal: Boolean);
begin
  if NewVal <> FChangeR then
  begin
    FChangeR := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeDrawG(NewVal: Boolean);
begin
  if NewVal <> FChangeG then
  begin
    FChangeG := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeDrawB(NewVal: Boolean);
begin
  if NewVal <> FChangeB then
  begin
    FChangeB := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeStartR(NewVal: Byte);
begin
  if FStartR <> NewVal then
  begin
    FStartR := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeStartG(NewVal: Byte);
begin
  if FStartG <> NewVal then
  begin
    FStartG := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeStartB(NewVal: Byte);
begin
  if FStartB <> NewVal then
  begin
    FStartB := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeDirection(NewVal: TFillDirection);
begin
  if NewVal <> FDirection then
  begin
    FDirection := NewVal;
    RefreshNeeded;
  end;
end;

procedure TBackdrop.ChangeEnabled(NewVal: Boolean);
begin
  If NewVal <> FEnabled then
  begin
    FEnabled := NewVal;
    if NewVal = True then RefreshNeeded;
  end;
end;

procedure TBackdrop.RefreshNeeded;
begin
  if Assigned(FRefreshEvent) then
    FRefreshEvent;
end;

end.
